// Generated by CoffeeScript 1.3.3
(function() {
  var Match, NoMatch, ParserState, Trampoline, util;

  util = require('util');

  Trampoline = require("./trampoline").Trampoline;

  ParserState = (function() {

    function ParserState(text, pos, end, lineno, xpos, trampoline, depth, _debugger) {
      this.text = text;
      this.pos = pos != null ? pos : 0;
      this.end = end;
      this.lineno = lineno != null ? lineno : 0;
      this.xpos = xpos != null ? xpos : 0;
      this.trampoline = trampoline != null ? trampoline : null;
      this.depth = depth != null ? depth : 0;
      this["debugger"] = _debugger != null ? _debugger : null;
      if (!(this.end != null)) {
        this.end = this.text.length;
      }
      if (!(this.trampoline != null)) {
        this.trampoline = new Trampoline(this);
      }
    }

    ParserState.prototype.toString = function() {
      var truncated;
      truncated = this.text.length > 10 ? "'" + this.text.slice(0, 10) + "...'" : "'" + this.text + "'";
      return "ParserState(text=" + truncated + ", pos=" + this.pos + ", end=" + this.end + ", lineno=" + this.lineno + ", xpos=" + this.xpos + ", depth=" + this.depth + ", work=" + (this.trampoline.size()) + ")";
    };

    ParserState.prototype.advance = function(n) {
      var lineno, pos, state, xpos;
      pos = this.pos;
      lineno = this.lineno;
      xpos = this.xpos;
      while (pos < this.pos + n) {
        if (this.text[pos++] === '\n') {
          lineno++;
          xpos = 0;
        } else {
          xpos++;
        }
      }
      state = new ParserState(this.text, pos, this.end, lineno, xpos, this.trampoline, this.depth, this["debugger"]);
      return state;
    };

    ParserState.prototype.line = function() {
      var end, lend, lstart, text;
      text = this.text;
      end = this.end;
      lstart = this.pos;
      lend = this.pos;
      if (lstart > 0 && text[lstart] === '\n') {
        lstart--;
      }
      while (lstart > 0 && text[lstart] !== '\n') {
        lstart--;
      }
      if (text[lstart] === '\n') {
        lstart++;
      }
      while (lend < end && text[lend] !== '\n') {
        lend++;
      }
      return text.slice(lstart, lend);
    };

    ParserState.prototype.fail = function(cont, message) {
      return cont(new NoMatch(this, "Expected " + message));
    };

    ParserState.prototype.getCache = function(parser) {
      return this.trampoline.getCache(parser, this);
    };

    ParserState.prototype.addJob = function(description, job) {
      return this.trampoline.push(this.depth, description, job);
    };

    ParserState.prototype.deeper = function() {
      return new ParserState(this.text, this.pos, this.end, this.lineno, this.xpos, this.trampoline, this.depth + 1, this["debugger"]);
    };

    ParserState.prototype.debug = function(f) {
      var lines;
      if (!this["debugger"]) {
        return;
      }
      lines = typeof f === 'function' ? f() : f;
      if (!Array.isArray(lines)) {
        lines = lines.split("\n");
      }
      return this.debugLines(lines);
    };

    ParserState.prototype.debugLines = function(lines) {
      var line, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = lines.length; _i < _len; _i++) {
        line = lines[_i];
        if (Array.isArray(line)) {
          _results.push(this.debugLines(line));
        } else {
          _results.push(this["debugger"](line));
        }
      }
      return _results;
    };

    return ParserState;

  })();

  Match = (function() {

    function Match(state, match, commit) {
      this.state = state;
      this.match = match;
      this.commit = commit != null ? commit : false;
      this.ok = true;
    }

    Match.prototype.toString = function() {
      return "Match(state=" + this.state + ", match=" + (util.inspect(this.match)) + ", commit=" + this.commit + ")";
    };

    Match.prototype.equals = function(other) {
      return other.ok && this.match === other.match && this.state.pos === other.state.pos;
    };

    return Match;

  })();

  NoMatch = (function() {

    function NoMatch(state, message, abort) {
      this.state = state;
      this.message = message;
      this.abort = abort != null ? abort : false;
      this.ok = false;
    }

    NoMatch.prototype.toString = function() {
      return "NoMatch(state=" + this.state + ", message='" + this.message + "', abort=" + this.abort + ")";
    };

    NoMatch.prototype.equals = function(other) {
      return (!other.ok) && this.state.pos === other.state.pos && this.message === other.message;
    };

    return NoMatch;

  })();

  exports.ParserState = ParserState;

  exports.Match = Match;

  exports.NoMatch = NoMatch;

}).call(this);
